import { Motion } from './Motion';
import { EasemateConfig } from './utils/Config';
export class Flow extends Motion {
    #children = [];
    #config;
    getInitialTargets() {
        return [];
    }
    constructor(config = {}, animate) {
        super(animate);
        this.#config = {
            duration: EasemateConfig.get('defaultDuration'),
            autoplay: EasemateConfig.get('autoplay'),
            ...config,
        };
        EasemateConfig.callPluginHooks('beforeInit', this, null, null, this.#config);
        EasemateConfig.callPluginHooks('init', this, null, null, this.#config);
        if (this.#config.autoplay) {
            this.play();
        }
        EasemateConfig.callPluginHooks('afterInit', this, null, null, this.#config);
        if (EasemateConfig.get('debug')) {
            console.log('Flow initialized with config:', this.#config);
        }
    }
    add(child) {
        if (!(child instanceof Motion) && EasemateConfig.get('debug')) {
            console.warn('Invalid child added to Flow:', child);
            return this;
        }
        this.#children.push(child);
        return this;
    }
    update(progress) {
        if (EasemateConfig.get('debug')) {
            console.log(`Flow updated with progress: ${progress}, children count: ${this.#children.length}`);
        }
        EasemateConfig.callPluginHooks('beforeUpdate', this, this.#children.filter((c) => c instanceof Motion).map((c) => c.update(0)), null, this.#config);
        const result = this.#children.filter((c) => c instanceof Motion).flatMap((child) => child.update(progress));
        EasemateConfig.callPluginHooks('afterUpdate', this, result, null, this.#config);
        return result;
    }
    play(duration = this.#config.duration ?? EasemateConfig.get('defaultDuration'), delay = this.#config.delay ?? 0) {
        super.play(duration, delay);
        this.#children.forEach((child) => child.play(duration, delay));
        if (EasemateConfig.get('debug')) {
            console.log('Flow playing with duration:', duration, 'delay:', delay);
        }
        return this;
    }
    pause() {
        super.pause();
        this.#children.forEach((child) => child.pause());
        if (EasemateConfig.get('debug')) {
            console.log('Flow paused, children count:', this.#children.length);
        }
        return this;
    }
    resume() {
        super.resume();
        this.#children.forEach((child) => {
            child.resume();
        });
        if (this.getId()) {
            const currentProgress = this.getState()?.progress ?? 0;
            this.setProgress(currentProgress);
        }
        if (EasemateConfig.get('debug')) {
            console.log('Flow resumed, children count:', this.#children.length);
        }
        return this;
    }
    kill() {
        super.kill();
        this.#children.forEach((child) => child.kill());
        this.#children = [];
        if (EasemateConfig.get('debug')) {
            console.log('Flow killed');
        }
        return this;
    }
    setProgress(progress) {
        super.setProgress(progress);
        this.#children.forEach((child) => child.setProgress(progress));
        return this;
    }
}
//# sourceMappingURL=Flow.js.map