import { clamp, interpolate, isArray, isNumber, isObject, isString, parseIncremental } from './utils/helpers';
import { EasemateConfig } from './utils/Config';
export class Modulate {
    interpolate = (from, to, progress) => {
        const clampedProgress = clamp(progress, 0, 1);
        EasemateConfig.callPluginHooks('beforeInterpolateRecursive', this, from, to, clampedProgress);
        const customInterpolator = EasemateConfig.getInterpolator(from) || interpolate;
        let result;
        if (isNumber(from) && isNumber(to)) {
            result = customInterpolator(from, to, clampedProgress);
        }
        else if (isNumber(from) && isString(to)) {
            const parsed = parseIncremental(from, to);
            result = customInterpolator(from, parsed, clampedProgress);
        }
        else if (isArray(from) && isArray(to)) {
            result = from.map((f, i) => this.interpolate(f, to[i], clampedProgress));
        }
        else if (isObject(from) && isObject(to)) {
            const resultObj = { ...from };
            for (const key in to) {
                resultObj[key] = this.interpolate(from[key], to[key], clampedProgress);
            }
            result = resultObj;
        }
        else {
            result = customInterpolator(from, to, clampedProgress);
        }
        EasemateConfig.callPluginHooks('afterInterpolateRecursive', this, from, to, clampedProgress, result);
        return result;
    };
    static register(type, interpolator) {
        EasemateConfig.registerInterpolator(type, interpolator);
    }
    static setDefaultInterpolator(interpolator) {
        EasemateConfig.registerInterpolator('default', interpolator);
    }
}
//# sourceMappingURL=Modulate.js.map