export type EasingFunction = (t: number) => number;
export type CoreEasings = 'linear' | 'ease' | 'easeIn' | 'easeOut' | 'easeInOut';
export type Interpolator<T> = (start: T, end: T, progress: number) => T;
export type CorePropsIncremental = `+=${number}` | `-=${number}` | `*=${number}`;
export type CoreBaseTarget = number | {
    [key: string]: number | CoreBaseTarget;
};
export type CoreArrayTarget = CoreBaseTarget[];
export type CoreTarget = CoreBaseTarget | CoreArrayTarget;
export type CoreProps<T> = T extends number ? number | CorePropsIncremental | ((index: number, initial: number, allInitials: number[]) => number) : T extends CoreBaseTarget ? {
    [K in keyof T]?: CoreProps<T[K]>;
} : T extends Array<infer U> ? Array<CoreProps<U>> : never;
export interface CoreConfig<T, P> {
    to?: P | ((index: number, initialVal: T, allInitials: T[]) => P);
    autoplay?: boolean;
    delay?: number;
    direction?: 'forward' | 'reverse' | 'return';
    duration?: number;
    easing?: CoreEasings | EasingFunction;
    repeat?: number;
    repeatDelay?: number;
    onStart?: (target: T, props: P, allTargets: T[]) => void;
    onUpdate?: (target: T, props: P, progress: number, allTargets: T[]) => void;
    onComplete?: (target: T, props: P, allTargets: T[]) => void;
}
export interface PluginTypes {
    core: {
        target: CoreTarget;
        props: CoreProps<CoreTarget>;
        config: CoreConfig<CoreTarget, CoreProps<CoreTarget>>;
    };
}
export type ExtendedTarget = {
    [K in keyof PluginTypes]: PluginTypes[K]['target'];
}[keyof PluginTypes];
export type GetPluginKey<T> = {
    [K in keyof PluginTypes]: T extends PluginTypes[K]['target'] ? K : never;
}[keyof PluginTypes];
export type GetProps<T> = PluginTypes[GetPluginKey<T>]['props'];
export type GetConfig<T> = PluginTypes[GetPluginKey<T>]['config'];
