export const isObject = (value) => {
    return typeof value === 'object' && value !== null && !Array.isArray(value);
};
export const isArray = (value) => {
    return Array.isArray(value);
};
export const isNumber = (value) => {
    return typeof value === 'number' && !Number.isNaN(value);
};
export const isFunction = (value) => {
    return typeof value === 'function';
};
export const isUndefined = (value) => {
    return typeof value === 'undefined';
};
export const isString = (value) => {
    return typeof value === 'string';
};
export const isNil = (value) => {
    return value === null || value === undefined;
};
export const mergeDeep = (target, source, immutable = true) => {
    const output = immutable ? { ...target } : target;
    if (isObject(target) && isObject(source)) {
        Object.keys(source).forEach((key) => {
            const targetValue = target[key];
            const sourceValue = source[key];
            if (isObject(sourceValue) && isObject(targetValue)) {
                output[key] = mergeDeep(targetValue, sourceValue, immutable);
            }
            else {
                output[key] = sourceValue;
            }
        });
    }
    return output;
};
export const lerp = (start, end, t) => start + (end - start) * t;
export const isBoolean = (value) => typeof value === 'boolean';
export const interpolate = (start, end, progress, interpolator) => {
    const clampedProgress = clamp(progress, 0, 1);
    if (interpolator) {
        return interpolator(start, end, clampedProgress);
    }
    if (isNumber(start) && isNumber(end)) {
        return (start + (end - start) * clampedProgress);
    }
    return clampedProgress >= 1 ? end : start;
};
export const clamp = (value, min, max) => {
    return Math.max(min, Math.min(max, value));
};
export const parseIncremental = (base, increment) => {
    const num = parseFloat(increment.slice(2));
    if (increment.startsWith('+=')) {
        return base + num;
    }
    if (increment.startsWith('-=')) {
        return base - num;
    }
    if (increment.startsWith('*=')) {
        return base * num;
    }
    return base;
};
export const normalize = (value, min, max) => {
    return clamp((value - min) / (max - min), 0, 1);
};
export const mapRange = (value, inMin, inMax, outMin, outMax) => {
    return outMin + (outMax - outMin) * normalize(value, inMin, inMax);
};
export const round = (value, decimals) => {
    if (!isNumber(decimals) || decimals <= 0) {
        return value;
    }
    const factor = Math.pow(10, decimals);
    return Math.round(value * factor) / factor;
};
//# sourceMappingURL=helpers.js.map