import type { EasingFunction } from './types';
interface AnimationOptions<T> {
    delay?: number;
    onStart?: (targets: T[]) => void;
    onUpdate?: (progress: number, targets: T[]) => void;
    onComplete?: (targets: T[]) => void;
}
interface AnimationDriver<T> {
    play: (targets: T[], keyframes: Keyframe<T>[], options: AnimationOptions<T>) => string;
    pause: (id: string) => void;
    resume: (id: string) => void;
    cancel: (id: string) => void;
    setProgress: (id: string, progress: number) => void;
    getState?: (id: string) => {
        progress: number;
        targets: T[];
        startTime: number;
    } | undefined;
    getActiveAnimationIds(): string[];
    isPaused(id: string): boolean;
}
export interface Keyframe<T> {
    value: T;
    duration: number;
    easing?: EasingFunction;
}
export declare class Animate<T> {
    #private;
    constructor(timeSource?: () => number);
    registerDriver(type: string, driver: AnimationDriver<T>): void;
    setDriver(typeOrDriver: string | AnimationDriver<T>): this;
    add(targets: T[], keyframes: Keyframe<T>[], options?: AnimationOptions<T>): string;
    pause(id: string): this;
    resume(id: string): this;
    cancel(id: string): this;
    setProgress(id: string, progress: number): this;
    getTargets(id: string): T[] | undefined;
    getState(id: string): {
        progress: number;
        targets: T[];
        startTime: number;
    } | undefined;
    getActiveAnimationIds(): string[];
    isPaused(id: string): boolean;
}
export {};
