"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Modulate = void 0;
const helpers_1 = require("./utils/helpers");
const Config_1 = require("./utils/Config");
class Modulate {
    interpolate = (from, to, progress) => {
        const clampedProgress = (0, helpers_1.clamp)(progress, 0, 1);
        Config_1.EasemateConfig.callPluginHooks('beforeInterpolateRecursive', this, from, to, clampedProgress);
        const customInterpolator = Config_1.EasemateConfig.getInterpolator(from) || helpers_1.interpolate;
        let result;
        if ((0, helpers_1.isNumber)(from) && (0, helpers_1.isNumber)(to)) {
            result = customInterpolator(from, to, clampedProgress);
        }
        else if ((0, helpers_1.isNumber)(from) && (0, helpers_1.isString)(to)) {
            const parsed = (0, helpers_1.parseIncremental)(from, to);
            result = customInterpolator(from, parsed, clampedProgress);
        }
        else if ((0, helpers_1.isArray)(from) && (0, helpers_1.isArray)(to)) {
            result = from.map((f, i) => this.interpolate(f, to[i], clampedProgress));
        }
        else if ((0, helpers_1.isObject)(from) && (0, helpers_1.isObject)(to)) {
            const resultObj = { ...from };
            for (const key in to) {
                resultObj[key] = this.interpolate(from[key], to[key], clampedProgress);
            }
            result = resultObj;
        }
        else {
            result = customInterpolator(from, to, clampedProgress);
        }
        Config_1.EasemateConfig.callPluginHooks('afterInterpolateRecursive', this, from, to, clampedProgress, result);
        return result;
    };
    static register(type, interpolator) {
        Config_1.EasemateConfig.registerInterpolator(type, interpolator);
    }
    static setDefaultInterpolator(interpolator) {
        Config_1.EasemateConfig.registerInterpolator('default', interpolator);
    }
}
exports.Modulate = Modulate;
//# sourceMappingURL=Modulate.js.map