"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Motion = void 0;
const Animate_1 = require("./Animate");
const Config_1 = require("./utils/Config");
class Motion {
    #animate;
    id = null;
    constructor(animate) {
        this.#animate = animate || new Animate_1.Animate();
    }
    play(duration = Config_1.EasemateConfig.get('defaultDuration'), delay = 0, keyframes) {
        if (this.id) {
            this.#animate.cancel(this.id);
        }
        this.update(0);
        const targets = this.id ? this.#animate.getTargets(this.id) || [] : this.getInitialTargets();
        this.id = this.#animate.add(targets, keyframes || [{ value: this.update(1)[0], duration }], {
            delay: delay,
            onStart: (targets) => {
                Config_1.EasemateConfig.callPluginHooks('start', this, targets, null, { duration, delay });
            },
            onUpdate: (progress, _targets) => {
                const updated = this.update(progress);
                Config_1.EasemateConfig.callPluginHooks('update', this, updated, null, { duration, delay });
            },
            onComplete: (targets) => {
                Config_1.EasemateConfig.callPluginHooks('complete', this, targets, null, { duration, delay });
            },
        });
        if (Config_1.EasemateConfig.get('debug')) {
            console.log(`Motion ${this.id} playing with duration: ${duration}ms, delay: ${delay}ms`);
        }
        return this;
    }
    resume() {
        if (this.id) {
            this.#animate.resume(this.id);
            const state = this.getState();
            const currentProgress = state?.progress ?? 0;
            const duration = state?.targets.length ? 1000 : Config_1.EasemateConfig.get('defaultDuration');
            const now = performance.now();
            const elapsed = currentProgress * duration;
            if (Config_1.EasemateConfig.get('debug')) {
                console.log(`Motion ${this.id} resumed at progress: ${currentProgress}, elapsed: ${elapsed}, now: ${now}`);
            }
        }
        return this;
    }
    pause() {
        if (this.id) {
            this.#animate.pause(this.id);
        }
        return this;
    }
    kill() {
        if (this.id) {
            this.#animate.cancel(this.id);
        }
        this.id = null;
        if (Config_1.EasemateConfig.get('debug')) {
            console.log(`Motion ${this.id || 'unknown'} killed`);
        }
        return this;
    }
    setProgress(progress) {
        if (this.id) {
            this.#animate.setProgress(this.id, progress);
        }
        return this;
    }
    getAnimate() {
        return this.#animate;
    }
    getId() {
        return this.id;
    }
    getState() {
        return this.id ? this.#animate.getState(this.id) : undefined;
    }
}
exports.Motion = Motion;
//# sourceMappingURL=Motion.js.map