import type { CoreConfig, CoreProps } from './types';
export interface PluginHooks<T = unknown, P = unknown, C = unknown> {
    onRegister?: () => void;
    onDispose?: () => void;
    beforeInit?: (target: T, props: P, config: C) => void;
    init?: (target: T, props: P, config: C) => void;
    afterInit?: (target: T, props: P, config: C) => void;
    beforeStart?: (target: T, props: P, config: C) => void;
    start?: (target: T, props: P, config: C) => void;
    afterStart?: (target: T, props: P, config: C) => void;
    beforeUpdate?: (target: T, props: P, config: C) => void;
    update?: (target: T, props: P, config: C) => void;
    afterUpdate?: (target: T, props: P, config: C) => void;
    apply?: (target: T, props: P, config: C) => void;
    tick?: (target: T, props: P, config: C) => void;
    beforeComplete?: (target: T, props: P, config: C) => void;
    complete?: (target: T, props: P, config: C) => void;
    afterComplete?: (target: T, props: P, config: C) => void;
    getInitialValues?: (target: T) => T | undefined;
    beforeInterpolateRecursive?: (fromVal: unknown, toVal: unknown, progress: number) => void;
    afterInterpolateRecursive?: (fromVal: unknown, toVal: unknown, progress: number, result: unknown) => void;
}
export interface PluginInterpolate<P = unknown> {
    value?: (start: number, end: number, progress: number) => number;
    props?: (props: P) => P;
}
export type EasematePlugin<T = unknown, P = unknown, C = unknown> = PluginHooks<T, P, C> & PluginInterpolate<P>;
export type CoreBaseTarget = number | {
    [key: string]: number | CoreBaseTarget;
};
export type CoreArrayTarget = CoreBaseTarget[];
export type CoreTarget = CoreBaseTarget | CoreArrayTarget;
export declare class Plugin<T extends CoreTarget | any = CoreTarget, P extends CoreProps<T> | any = CoreProps<T>, C extends CoreConfig<T, P> | any = CoreConfig<T, P>> {
    #private;
    readonly id: string;
    constructor(id: string, hooks?: PluginHooks<T, P, C>, interpolateConfig?: PluginInterpolate<P>);
    get enabled(): boolean;
    enable(): this;
    disable(): this;
    get priority(): number;
    setPriority(priority: number): this;
    get dependencies(): string[];
    setDependencies(...pluginIds: string[]): this;
    addDependency(pluginId: string): this;
    removeDependency(pluginId: string): this;
    hasDependency(pluginId: string): boolean;
    getHooks(): PluginHooks<T, P, C>;
    getInterpolate(): PluginInterpolate<P>;
    validateDependencies(registeredPlugins: Plugin<T, P, C>[]): boolean;
    static validatePluginChain(plugins: Plugin[]): Plugin[];
}
