"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EasemateConfig = exports.PluginHookRegistry = void 0;
const helpers_1 = require("./helpers");
class PluginHookRegistry {
    static hooks = [];
    static register(plugin) {
        this.hooks.push(plugin);
    }
    static clear() {
        this.hooks = [];
    }
    static getAll() {
        return this.hooks;
    }
}
exports.PluginHookRegistry = PluginHookRegistry;
class EasemateConfig {
    static #config = {};
    static #interpolateRegistry = new Map();
    static #defaultConfig = {
        pauseOnHidden: false,
        rate: 1,
        debug: false,
        precision: null,
        defaultDuration: 1000,
        defaultEase: 'ease',
        autoplay: true,
        smoothing: false,
        useSubFrames: false,
        maxFps: 120,
        smoothingFactor: 0.5,
    };
    static get(key) {
        return (EasemateConfig.#config[key] ?? EasemateConfig.#defaultConfig[key]);
    }
    static set(key, value) {
        EasemateConfig.#config[key] = value;
    }
    static getAll() {
        return { ...EasemateConfig.#config };
    }
    static reset(config) {
        if (config) {
            EasemateConfig.#config = { ...EasemateConfig.#defaultConfig, ...config };
        }
        else {
            EasemateConfig.#config = { ...EasemateConfig.#defaultConfig };
        }
    }
    static registerInterpolator(type, interpolator) {
        EasemateConfig.#interpolateRegistry.set(type, interpolator);
    }
    static getInterpolator(type) {
        return EasemateConfig.#interpolateRegistry.get(type);
    }
    static callPluginHooks(hookName, instance, target, props, config, result) {
        for (const plugin of PluginHookRegistry.getAll()) {
            const hook = plugin[hookName];
            if (hook && (0, helpers_1.isFunction)(hook)) {
                if (result !== undefined) {
                    hook.call(instance, target, props, config, result);
                }
                else {
                    hook.call(instance, target, props, config);
                }
            }
        }
    }
}
exports.EasemateConfig = EasemateConfig;
//# sourceMappingURL=Config.js.map