"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.round = exports.mapRange = exports.normalize = exports.parseIncremental = exports.clamp = exports.interpolate = exports.isBoolean = exports.lerp = exports.mergeDeep = exports.isNil = exports.isString = exports.isUndefined = exports.isFunction = exports.isNumber = exports.isArray = exports.isObject = void 0;
const isObject = (value) => {
    return typeof value === 'object' && value !== null && !Array.isArray(value);
};
exports.isObject = isObject;
const isArray = (value) => {
    return Array.isArray(value);
};
exports.isArray = isArray;
const isNumber = (value) => {
    return typeof value === 'number' && !Number.isNaN(value);
};
exports.isNumber = isNumber;
const isFunction = (value) => {
    return typeof value === 'function';
};
exports.isFunction = isFunction;
const isUndefined = (value) => {
    return typeof value === 'undefined';
};
exports.isUndefined = isUndefined;
const isString = (value) => {
    return typeof value === 'string';
};
exports.isString = isString;
const isNil = (value) => {
    return value === null || value === undefined;
};
exports.isNil = isNil;
const mergeDeep = (target, source, immutable = true) => {
    const output = immutable ? { ...target } : target;
    if ((0, exports.isObject)(target) && (0, exports.isObject)(source)) {
        Object.keys(source).forEach((key) => {
            const targetValue = target[key];
            const sourceValue = source[key];
            if ((0, exports.isObject)(sourceValue) && (0, exports.isObject)(targetValue)) {
                output[key] = (0, exports.mergeDeep)(targetValue, sourceValue, immutable);
            }
            else {
                output[key] = sourceValue;
            }
        });
    }
    return output;
};
exports.mergeDeep = mergeDeep;
const lerp = (start, end, t) => start + (end - start) * t;
exports.lerp = lerp;
const isBoolean = (value) => typeof value === 'boolean';
exports.isBoolean = isBoolean;
const interpolate = (start, end, progress, interpolator) => {
    const clampedProgress = (0, exports.clamp)(progress, 0, 1);
    if (interpolator) {
        return interpolator(start, end, clampedProgress);
    }
    if ((0, exports.isNumber)(start) && (0, exports.isNumber)(end)) {
        return (start + (end - start) * clampedProgress);
    }
    return clampedProgress >= 1 ? end : start;
};
exports.interpolate = interpolate;
const clamp = (value, min, max) => {
    return Math.max(min, Math.min(max, value));
};
exports.clamp = clamp;
const parseIncremental = (base, increment) => {
    const num = parseFloat(increment.slice(2));
    if (increment.startsWith('+=')) {
        return base + num;
    }
    if (increment.startsWith('-=')) {
        return base - num;
    }
    if (increment.startsWith('*=')) {
        return base * num;
    }
    return base;
};
exports.parseIncremental = parseIncremental;
const normalize = (value, min, max) => {
    return (0, exports.clamp)((value - min) / (max - min), 0, 1);
};
exports.normalize = normalize;
const mapRange = (value, inMin, inMax, outMin, outMax) => {
    return outMin + (outMax - outMin) * (0, exports.normalize)(value, inMin, inMax);
};
exports.mapRange = mapRange;
const round = (value, decimals) => {
    if (!(0, exports.isNumber)(decimals) || decimals <= 0) {
        return value;
    }
    const factor = Math.pow(10, decimals);
    return Math.round(value * factor) / factor;
};
exports.round = round;
//# sourceMappingURL=helpers.js.map